﻿//
//  PDF Structure
//  Metadata sample code
//
/*
Copyright(c) 2024 株式会社トラスト・ソフトウェア・システム, All Rights Reserved.
*/

using System;
using System.Security.Cryptography.X509Certificates;
using PDFTools.PdfStructure;
using PDFTools.PdfStructure.Security;
using PDFTools.PdfStructure.Security.Misc;

namespace Metadata
{
    class Metadata
    {
        static void Main(string[] args)
        {
            using (var stc = new Structure())
            {
                //PDFオープン
                stc.OpenDoc("../../../../../resources/input.pdf");

                // XMPインターフェース
                XmpInterface xmp = stc.GetXmpInterface();

                //独自のSimpleプロパティに取引日・取引先・取引金額を追加する
                const string e = "http://sample.com/Archived/";
                const string s = "Arch";
                string val;
                xmp.RegisterNamespace(e, s, out val);

                //プロパティ追加
                xmp.SetSimplePropertyCurrentDate(e, "取引日");
                xmp.SetSimpleProperty(e, "取引先", "株式会社サンプル");
                xmp.SetSimpleProperty(e, "取引金額", "123,456,000.-");

                //メタデータ更新
                xmp.UpdateDocument();

                //Securityインターフェース取得
                var sec = stc.GetSecurityInterface();

                //タイムスタンプ局指定
                sec.TimeStamp = "http://tsa.trustss.jp/";

                //出力ファイル指定
                sec.SetOutputFileName("out.pdf");

                //電子署名と格納（Securityインターフェース終了）
                sec.SignCloseInterface();

                xmp.CloseInterface();
            }
        }
    }
}
